@@ -1,26 +1,40 @@
 from __future__ import annotations
 import json
-import re
 import sys
-from inspect import getsource
-from typed_cap.parser import Parser, ParserRegister, PRESET as PRESET_PARSERS
 from typed_cap.types import (
+    ArgNamed,
     ArgOpt,
+    ArgTypes,
     ArgsParserKeyError,
     ArgsParserMissingArgument,
     ArgsParserMissingValue,
+    ArgsParserOptions,
     ArgsParserUndefinedParser,
     ArgsParserUnexpectedValue,
+    CapArgKeyNotFound,
+    CapInvalidAlias,
+    CapInvalidDefaultValue,
+    CapInvalidValue,
+    CapUnknownArg,
     Unhandled,
     VALID_ALIAS_CANDIDATES,
 )
+from typed_cap.args_parser import args_parser
+from typed_cap.typing import (
+    VALIDATOR,
+    AnnoExtra,
+    get_optional_candidates,
+    get_queue_type,
+    get_type_candidates,
+    typpeddict_parse,
+    typpeddict_parse_extra,
+)
 from typed_cap.utils import (
-    args_parser,
     flatten,
     get_terminal_width,
     panic,
-    remove_comments,
     split_by_length,
+    to_red,
     to_yellow,
     to_blue,
     unwrap_or,
@@ -31,9 +45,10 @@ from typing import (
     Dict,
     Generic,
     List,
-    NamedTuple,
+    Literal,
     NoReturn,
     Optional,
+    Protocol,
     Tuple,
     Type,
     TypeVar,
@@ -47,10 +62,9 @@ ArgCallback = Callable[["Cap", List[List]], Union[NoReturn, List[List]]]
 
 class _ArgOpt(TypedDict):
     val: Optional[Any]
-    type: str
+    type: Type
     about: Optional[str]
     alias: Optional[VALID_ALIAS_CANDIDATES]
-    optional: bool
     cb: Optional[ArgCallback]
     cb_idx: int
     hide: bool
@@ -59,7 +73,7 @@ class _ArgOpt(TypedDict):
 class _ParsedVal(TypedDict):
     val: List[List[Any]]
     default_val: Optional[Any]
-    is_list: bool
+    queue_type: Optional[Literal["list", "tuple"]]
 
 
 T = TypeVar("T", bound=TypedDict)
@@ -91,8 +105,12 @@ class Parsed(Generic[T]):
             pv = flatten(pv)
             if len(pv) == 0:
                 val[key] = parsed["default_val"]
+            elif parsed["queue_type"] == "list":
+                val[key] = flatten(pv)
+            elif parsed["queue_type"] == "tuple":
+                val[key] = pv[-1]
             else:
-                val[key] = pv if parsed["is_list"] else pv[-1]
+                val[key] = pv[-1]
         return val  # type: ignore
 
     @property
@@ -101,7 +119,7 @@ class Parsed(Generic[T]):
 
     def count(self, name: str) -> int:
         parsed = self._parsed_map.get(name)
-        if parsed != None:
+        if parsed is not None:
             return len(parsed["val"])
         else:
             panic(f'Parsed.count: cannot find option with name "{name}"')
@@ -125,6 +143,9 @@ CAP_ERR = Union[
     ArgsParserMissingValue,
     ArgsParserUndefinedParser,
     ArgsParserUnexpectedValue,
+    CapInvalidDefaultValue,
+    CapInvalidValue,
+    CapUnknownArg,
     Unhandled,
 ]
 
@@ -133,7 +154,7 @@ def _helper_help_cb(c: "Cap", v: List[List[bool]]) -> NoReturn:
     lns: List[Tuple[int, str]] = []
     INDENT_SIZE = 4
     if v[0][0]:
-        if c._about != None:
+        if c._about is not None:
             lns.append((0, c._about))
             lns.append((0, ""))
         lns.append((0, "OPTIONS:"))
@@ -173,7 +194,7 @@ def _helper_help_cb(c: "Cap", v: List[List[bool]]) -> NoReturn:
 def _helper_version_cb(c: "Cap", v: List[List[bool]]) -> NoReturn:
     if v[0][0]:
         ver = unwrap_or(c._version, "unknown version")
-        if c._name != None:
+        if c._name is not None:
             print(f"{c._name} {ver}")
         else:
             print(ver)
@@ -183,40 +204,54 @@ def _helper_version_cb(c: "Cap", v: List[List[bool]]) -> NoReturn:
 def helper_arg_help(
     cap: "Cap",
     name: str = "help",
-    alias: Optional[VALID_ALIAS_CANDIDATES] = "h",
+    alias: Optional[VALID_ALIAS_CANDIDATES] = None,
 ):
     cap.add_argument(
         name,
-        type="bool",
+        arg_type=Optional[bool],
         about="display the help text",
-        alias=alias,
-        optional=True,
+        alias=alias if alias is not None else "h",
         callback=_helper_help_cb,
         callback_priority=0,
+        hide=True,
         prevent_overwrite=True,
+        ignore_invalid_alias=False if alias is not None else True,
     )
+    cap._preset_helper_used = True
 
 
 def helper_arg_version(
     cap: "Cap",
     name: str = "version",
-    alias: Optional[VALID_ALIAS_CANDIDATES] = "V",
+    alias: Optional[VALID_ALIAS_CANDIDATES] = None,
 ):
     cap.add_argument(
         name,
-        type="bool",
+        arg_type=Optional[bool],
         about="print version info and exit",
-        alias=alias,
-        optional=True,
+        alias=alias if alias is not None else "V",
         callback=_helper_version_cb,
         callback_priority=2,
+        hide=True,
         prevent_overwrite=True,
+        ignore_invalid_alias=False if alias is not None else True,
     )
+    cap._preset_helper_used = True
+
+
+class _Helper_fn(Protocol):
+    def __call__(
+        self,
+        cap: "Cap",
+        name: str,
+        alias: Optional[VALID_ALIAS_CANDIDATES] = None,
+    ) -> None:
+        pass
 
 
 class _Helpers(TypedDict):
-    arg_help: Callable[["Cap", str, Optional[VALID_ALIAS_CANDIDATES]], None]
-    arg_version: Callable[["Cap", str, Optional[VALID_ALIAS_CANDIDATES]], None]
+    arg_help: _Helper_fn
+    arg_version: _Helper_fn
 
 
 helpers: _Helpers = {
@@ -225,151 +260,147 @@ helpers: _Helpers = {
 }
 
 
+def colorize_text_t_type(t: Type) -> str:
+    tn = ""
+    try:
+        tn = t.__name__
+    except AttributeError:
+        tn = str(t)
+    return to_blue(tn)
+
+
+def colorize_text_t_option_name(key: str) -> str:
+    return to_yellow(key)
+
+
+def colorize_text_t_value(val: Any) -> str:
+    try:
+        return to_red(str(val))
+    except Exception as err:
+        print(err)
+        return "[...]"
+        # TODO:
+
+
 class Cap(Generic[K, T, U]):
     _argstype: Type[T]
     _args: Dict[str, _ArgOpt]
     _delimiter: Optional[str] = ","
-    _parser_register: ParserRegister
     _about: Optional[str]
     _name: Optional[str]
     _version: Optional[str]
     _raw_err: bool
+    _preset_helper_used: bool
 
     def __init__(
         self,
         argstype: Type[T],
     ) -> None:
         self._argstype = argstype
-        self._parser_register = PRESET_PARSERS
-        self._parse_argstype()
+        self._args = {}
         self._about = None
         self._name = None
         self._version = None
         self._raw_err = False
+        self._preset_helper_used = False
+        self._parse_argstype()
 
-    def _get_key(
-        self, name: str
-    ) -> Union[NoReturn, str,]:
+    def _get_key(self, name: str) -> Union[NoReturn, str]:
         for key, opt in self._args.items():
             if key == name:
                 return key
             if opt["alias"] == name:
                 return key
-        panic(f"key '{name}' not found")
+        raise CapArgKeyNotFound(name)
 
-    def _extract_list_type(self, t: str) -> Tuple[bool, str]:
-        if t == "list":
-            return True, "str"
-        reg = re.compile(r"^(list\[(?P<type>\w+)\])$")
-        m = reg.match(t)
-        if m != None:
-            return True, m.group("type")
+    def _set_alias(
+        self, key: str, alias: Optional[str]
+    ) -> Union[NoReturn, None]:
+        opt = self._args.get(key)
+        if opt is None:
+            raise CapArgKeyNotFound(key)
         else:
-            return False, t
+            if alias is not None:
+                try:
+                    self._get_key(alias)
+                    raise CapInvalidAlias(key, alias)
+                except CapArgKeyNotFound:
+                    self._args[key] = {**opt, **{"alias": alias}}  # type: ignore
+            else:
+                self._args[key] = {**opt, **{"alias": None}}  # type: ignore
 
     def _panic(self, msg: str, alt_title: str, err: CAP_ERR) -> NoReturn:
         if self._raw_err:
             raise err
         else:
             title = unwrap_or(self._name, alt_title)
-            err_msg = f"{title}: {msg}"
+            err_msg = f"{title}: {msg}\n\t{err.__class__.__name__}"
             panic(err_msg)
 
     def _parse_argstype(self):
-        types = remove_comments(getsource(self._argstype))[1:]
-        self._args = {}
-        reg = re.compile(r"^(optional)\[(?P<type>.+)\]$")
-        for t in types:
-            t = t.split(":")
-            if len(t) != 2:
-                if len(t) == 1:
-                    if t[0].lstrip() == "":
-                        continue
-                    if t[0].lstrip()[:4] == "pass":
-                        continue
-                raise Exception(
-                    "failed to parse given types in TypedDict"
-                )  # TODO:
-            t = [x.lstrip() for x in t]
-            t = [x.rstrip() for x in t]
-            key, t = t
-            opt: bool = False
-            t = t.lower()
-            m = reg.match(t)
-            if m != None:
-                opt = True
-                t = m.group("type")
-            self._args[key] = {
-                "val": None,
-                "type": t,
-                "about": None,
-                "alias": None,
-                "optional": opt,
-                "cb": None,
-                "cb_idx": 0,
-                "hide": False,
-            }
+        typed: Dict[str, Type]
+        extra: Optional[Dict[str, AnnoExtra]] = None
+        if sys.version_info.minor < 9:
+            typed = typpeddict_parse(self._argstype)
+        else:
+            typed, extra = typpeddict_parse_extra(self._argstype)
+
+        for key, t in typed.items():
+            self.add_argument(
+                key,
+                arg_type=t,
+                callback_priority=0,
+                hide=False,
+                prevent_overwrite=False,
+                ignore_invalid_alias=False,
+            )
+        if extra is not None:
+            _ext: Dict[K, AnnoExtra] = extra  # type: ignore
+            self.helper({k: v.to_helper() for k, v in _ext.items()})
 
     def add_argument(
         self,
         key: str,
-        type: Union[str, Type[str], Type[int], Type[float], Type[bool]],
+        arg_type: Type,
         about: Optional[str] = None,
         alias: Optional[VALID_ALIAS_CANDIDATES] = None,
-        optional: bool = False,
         default: Optional[Any] = None,
         callback: Optional[ArgCallback] = None,
         callback_priority: int = 1,
+        hide: bool = False,
         prevent_overwrite: bool = False,
+        ignore_invalid_alias: bool = False,
     ) -> Cap:
-        if self._args.get(key) != None and prevent_overwrite:
+        if self._args.get(key) is not None and prevent_overwrite:
+            # TODO: sending any message?
             return self
-        else:
-            t: str
-            if isinstance(type, str):
-                t = type
-            else:
-                if type == str:
-                    t = "str"
-                elif type == int:
-                    t = "int"
-                elif type == float:
-                    t = "float"
-                elif type == bool:
-                    t = "bool"
+        self._args[key] = {
+            "val": default,
+            "type": arg_type,
+            "about": about,
+            "alias": None,
+            "cb": callback,
+            "cb_idx": callback_priority,
+            "hide": hide,
+        }
+        if alias is not None:
+            try:
+                self._set_alias(key, alias)
+            except CapInvalidAlias as err:
+                if ignore_invalid_alias:
+                    pass
                 else:
-                    raise Unhandled(
-                        desc="cannot convert given type to string",
-                        loc="Cap.add_argument",
-                    )
-            self._args[key] = {
-                "val": default,
-                "type": t,
-                "about": about,
-                "alias": alias,
-                "optional": optional,
-                "cb": callback,
-                "cb_idx": callback_priority,
-                "hide": True,
-            }
-            return self
+                    raise err
+        return self
 
     def set_delimiter(self, delimiter: Optional[str]) -> Cap:
         self._delimiter = delimiter
         return self
 
-    def set_parser(
-        self, type_name: str, parser: Parser, allow_list: bool
-    ) -> Cap:
-        self._parser_register.__setitem__(
-            type_name, {"parser": parser, "allow_list": allow_list}
-        )
-        return self
-
     def set_callback(
         self, key: str, callback: ArgCallback, priority: int = 1
     ) -> Cap:
-        if self._args.get(key) != None:
+        if self._args.get(key) is not None:
             self._args[key]["cb"] = callback
             self._args[key]["cb_idx"] = priority
         else:
@@ -403,48 +434,98 @@ class Cap(Generic[K, T, U]):
 
     def default_strict(self, value: T) -> Cap:
         for arg, val in value.items():
-            self._args[arg]["val"] = val
+            try:
+                t = self._args[arg]["type"]
+                valid, _, _ = VALIDATOR.extract(t, val, cvt=False)
+                if valid:
+                    self._args[arg]["val"] = val
+                else:
+                    # raise CapInvalidDefaultValue(arg, t, val)
+                    self._panic(
+                        f"invalid default value {colorize_text_t_value(val)} for option {colorize_text_t_option_name(arg)}:{colorize_text_t_type(t)}",
+                        "Cap.default_strict",
+                        CapInvalidDefaultValue(arg, t, val),
+                    )
+            except KeyError as err:
+                name = str(err)
+                self._panic(
+                    f"unknown named argument {colorize_text_t_option_name(name)}",
+                    "Cap.default_strict",
+                    CapUnknownArg(name, "default_strict"),
+                )
+            except Exception as err:
+                raise Unhandled(
+                    desc=f"unknown issue: {err.__class__.__name__}",
+                    loc="Cap.default_strict",
+                )
         return self
 
     def helper(self, helpers: Dict[K, ArgOpt]) -> Cap:
+        if self._preset_helper_used:
+            print(
+                "[warn] detected call of `Cap.helper` after call of preset helpers"
+            )
         for arg, opt in helpers.items():
-            self._args[arg] = {**self._args[arg], **opt}  # type: ignore[misc]
+            try:
+                try:
+                    alias = opt.pop("alias")
+                    self._set_alias(arg, alias)
+                except KeyError:
+                    pass
+                self._args[arg] = {**self._args[arg], **opt}  # type: ignore[misc]
+
+            except KeyError as err:
+                name = str(err)
+                self._panic(
+                    f"unknown named argument {colorize_text_t_option_name(name)}",
+                    "Cap.helper",
+                    CapUnknownArg(name, "helper"),
+                )
+            except CapInvalidAlias as err:
+                raise err
+            except Exception as err:
+                raise Unhandled(
+                    desc=f"unknown issue: {err.__class__.__name__}",
+                    loc="Cap.helper",
+                )
         return self
 
     def parse(
         self,
-        args: List[str] = sys.argv[1:],
-        ignore_unknown: bool = False,
-        ignore_unknown_flags: bool = False,
-        ignore_unknown_options: bool = False,
+        argv: List[str] = sys.argv[1:],
+        args_parser_options: Optional[ArgsParserOptions] = None,
     ) -> Parsed[T]:
+        def _is_flag(t: Type) -> bool:
+            if t == bool:
+                return True
+            try:
+                can = get_type_candidates(t)
+                return bool in can
+            except Exception:
+                return False
 
-        flags: List[Tuple[str, Optional[str]]] = []
-        options: List[Tuple[str, Optional[str]]] = []
+        named_args: List[Tuple[ArgTypes, ArgNamed]] = []
         for key, opt in self._args.items():
-            if opt["type"] == "bool":
-                flags.append((key, opt["alias"]))
-            else:
-                options.append((key, opt["alias"]))
+            named_args.append(
+                (
+                    "flag" if _is_flag(opt["type"]) else "option",
+                    (key, opt["alias"]),
+                )
+            )
 
         try:
-            out = args_parser(
-                args,
-                flags,
-                options,
-                ignore_unknown=ignore_unknown,
-                ignore_unknown_flags=ignore_unknown_flags,
-                ignore_unknown_options=ignore_unknown_options,
-            )
+            out = args_parser(argv, named_args, args_parser_options)
         except ArgsParserKeyError as err:
             self._panic(
-                f"unknown {err.key_type} '{err.key}'", "Cap.parse", err
+                f"unknown {err.key_type} {colorize_text_t_option_name(err.key)}",
+                "Cap.parse",
+                err,
             )
         except ArgsParserUnexpectedValue as err:
             key = self._get_key(err.key)
             prefix = "--"
             self._panic(
-                f"the value for argument '{to_yellow(prefix + key)}' wasn't expected",
+                f"the value for argument {colorize_text_t_option_name(prefix + key)} wasn't expected",
                 "Cap.parse",
                 err,
             )
@@ -452,58 +533,58 @@ class Cap(Generic[K, T, U]):
             key = self._get_key(err.key)
             prefix = "--"
             self._panic(
-                f"the argument '{to_yellow(prefix+key)}' requires a value, which was not supplied",
+                f"the argument {colorize_text_t_option_name(prefix+key)} requires a value, which was not supplied",
                 "Cap.parse",
                 err,
             )
 
         parsed_map: Dict[str, _ParsedVal] = {}
-        for key, val in out["options"].items():
-            opt = self._args[self._get_key(key)]  # TODO:
-            is_list, t = self._extract_list_type(opt["type"])
-            _val: List[List[Any]] = []
+        # extract process
+        for name, val in out["options"].items():
+            key = self._get_key(name)
+            parsed: _ParsedVal = parsed_map.get(
+                key,
+                {
+                    "val": [],
+                    "default_val": None,
+                    "queue_type": None,
+                },
+            )
+            opt = self._args[key]  # TODO:
+            parsed["queue_type"] = get_queue_type(
+                opt["type"], allow_optional=True
+            )
             for v in val:
-                if isinstance(v, bool):
-                    _val.append([v])
-                    continue
-                parser_inf = self._parser_register.get(t)
-                if parser_inf == None:
-                    self._panic(
-                        f"cannot handle type {to_blue(t)} because it has not been registered yet",
-                        "Cap.parse",
-                        ArgsParserUndefinedParser(t),
-                    )
+                t = opt["type"]
+                valid, v_got, err = VALIDATOR.extract(t, v, cvt=True)
+                # TODO: catch extract failed
+                if valid:
+                    parsed["val"].append([v_got])
                 else:
-                    _val.append(
-                        parser_inf["parser"](
-                            v,
-                            is_list and parser_inf["allow_list"],
-                            self._delimiter,
-                        )
+                    self._panic(
+                        f"invalid value {colorize_text_t_value(v)} for option {colorize_text_t_option_name(key)}:{colorize_text_t_type(t)}",
+                        "Cap.default_strict",
+                        CapInvalidValue(key, t, v),
                     )
-            parsed_map[self._get_key(key)] = {  # TODO:
-                "val": _val,
-                "is_list": is_list,
-                "default_val": None,
-            }
+            parsed_map[key] = parsed
 
         # callbacks
         cb_list: List[Tuple[str, int]] = []
         for key, opt in self._args.items():
-            if opt["cb"] != None:
+            if opt["cb"] is not None:
                 cb_list.append((key, opt["cb_idx"]))
         cb_list = sorted(cb_list, key=lambda x: x[1])
         cb_list.reverse()
         for key, _ in cb_list:
             parsed = parsed_map.get(key)
-            if parsed == None:
+            if parsed is None:
                 continue
             else:
                 if len(parsed["val"]) >= 0:
                     try:
                         arg = self._args[key]
                         cb = arg["cb"]
-                        if cb != None:
+                        if cb is not None:
                             parsed_map[key]["val"] = cb(self, parsed["val"])
                     except KeyError:
                         continue
@@ -511,21 +592,27 @@ class Cap(Generic[K, T, U]):
         # assign default value to empty field
         for key, opt in self._args.items():
             if opt["hide"]:
-                if parsed_map.get(key) != None:
+                if parsed_map.get(key) is not None:
                     parsed_map.pop(key)
             else:
-                if parsed_map.get(key) == None:
-                    is_list, t = self._extract_list_type(opt["type"])
-                    if opt["val"] == None and not opt["optional"]:
+                if parsed_map.get(key) is None:
+                    parsed_map[key] = {
+                        "val": [],
+                        "default_val": opt[
+                            "val"
+                        ],  # TODO: checking typeof default value
+                        "queue_type": get_queue_type(
+                            opt["type"], allow_optional=True
+                        ),
+                    }
+                    if (
+                        opt["val"] is None
+                        and get_optional_candidates(opt["type"]) is None
+                    ):
                         self._panic(
-                            f"option '{to_yellow(key)}':{to_blue(opt['type'])} is required but it is missing",
+                            f"option {colorize_text_t_option_name(key)}:{colorize_text_t_type(opt['type'])} is required but it is missing",
                             "Cap.parse",
                             ArgsParserMissingArgument(key, opt["type"]),
                         )
-                    parsed_map[key] = {
-                        "val": [],
-                        "is_list": is_list,
-                        "default_val": opt["val"],
-                    }
 
         return Parsed(out["args"], parsed_map)
